@@ -345,6 +345,7 @@ void indicateparams(genericptr thiselem)
 	    k = epp->pdata.pathpt[1];
 	 else
 	    k = epp->pdata.pointno;
+	 if (k < 0) k = 0;
 	 switch(ops->which) {
 	    case P_POSITION: case P_POSITION_X: case P_POSITION_Y:
 	       switch(thiselem->type) {
@@ -364,7 +365,10 @@ void indicateparams(genericptr thiselem)
 	             UDrawCircle(&TOSPLINE(&thiselem)->ctrl[k], ops->which);
 		     break;
 		  case PATH:
-		     pgen = ((pathptr)thiselem)->plist + epp->pdata.pathpt[0];
+		     if (epp->pdata.pathpt[0] < 0)
+		        pgen = ((pathptr)thiselem)->plist;
+		     else
+		        pgen = ((pathptr)thiselem)->plist + epp->pdata.pathpt[0];
 		     if (ELEMENTTYPE(*pgen) == POLYGON)
 	                 UDrawCircle(TOPOLY(pgen)->points + k, ops->which);
 		     else	/* spline */
@@ -549,7 +553,7 @@ int epsubstitute(genericptr thiselem, objectptr thisobj, objinstptr pinst,
    eparamptr epp;
    oparamptr dps, ops;
    int retval = -1;
-   int k, ival;
+   int i, k, ival, diff;
    char *key;
    float fval;
    XPoint *setpt;
@@ -641,22 +645,70 @@ int epsubstitute(genericptr thiselem, objectptr thisobj, objinstptr pinst,
 	    retval = max(retval, 1);
 	    switch(thiselem->type) {
 	       case PATH:
-		  pgen = TOPATH(&thiselem)->plist + epp->pdata.pathpt[0];
-		  if (ELEMENTTYPE(*pgen) == POLYGON) {
-		     setpt = TOPOLY(pgen)->points + k;
-		     setpt->x = ival;
+		  if (k < 0) {
+		     pgen = TOPATH(&thiselem)->plist;
+		     if (ELEMENTTYPE(*pgen) == POLYGON) {
+		        setpt = TOPOLY(pgen)->points;
+		        diff = ival - setpt->x;
+		     }
+		     else { /* spline */
+		        diff = ival - TOSPLINE(pgen)->ctrl[0].x;
+		     }
+		     for (pgen = TOPATH(&thiselem)->plist; pgen <
+				TOPATH(&thiselem)->plist +
+				TOPATH(&thiselem)->parts; pgen++) {
+		        if (ELEMENTTYPE(*pgen) == POLYGON) {
+			   for (i = 0; i < TOPOLY(pgen)->number; i++) {
+		              setpt = TOPOLY(pgen)->points + i;
+		              setpt->x += diff;
+			   }
+		        }
+		        else { /* spline */
+			   for (i = 0; i < 4; i++) {
+		              TOSPLINE(pgen)->ctrl[i].x += diff;
+			   }
+		           if (needrecalc) *needrecalc = True;
+		        }
+		     }
 		  }
-		  else { /* spline */
-		     TOSPLINE(pgen)->ctrl[k].x = ival;
-		     if (needrecalc) *needrecalc = True;
+		  else {
+		     pgen = TOPATH(&thiselem)->plist + epp->pdata.pathpt[0];
+		     if (ELEMENTTYPE(*pgen) == POLYGON) {
+		        setpt = TOPOLY(pgen)->points + k;
+		        setpt->x = ival;
+		     }
+		     else { /* spline */
+		        TOSPLINE(pgen)->ctrl[k].x = ival;
+		        if (needrecalc) *needrecalc = True;
+		     }
 		  }
 		  break;
 	       case POLYGON:
-	          setpt = TOPOLY(&thiselem)->points + k;
-		  setpt->x = ival;
+		  if (k < 0) {
+		     setpt = TOPOLY(&thiselem)->points;
+		     diff = ival - setpt->x;
+		     for (i = 0; i < TOPOLY(&thiselem)->number; i++) {
+			setpt = TOPOLY(&thiselem)->points + i;
+			setpt->x += diff;
+		     }
+		  }
+		  else {
+	             setpt = TOPOLY(&thiselem)->points + k;
+		     setpt->x = ival;
+		  }
 		  break;
 	       case SPLINE:
-		  TOSPLINE(&thiselem)->ctrl[k].x = ival;
+		  if (k < 0) {
+		     setpt = &(TOSPLINE(&thiselem)->ctrl[0]);
+		     diff = ival - setpt->x;
+		     for (i = 0; i < 4; i++) {
+			setpt = &(TOSPLINE(&thiselem)->ctrl[i]);
+			setpt->x += diff;
+		     }
+		  }
+		  else {
+		     TOSPLINE(&thiselem)->ctrl[k].x = ival;
+		  }
 		  if (needrecalc) *needrecalc = True;
 		  break;
 	       case LABEL:
@@ -674,22 +726,70 @@ int epsubstitute(genericptr thiselem, objectptr thisobj, objinstptr pinst,
 	    retval = max(retval, 1);
 	    switch(thiselem->type) {
 	       case PATH:
-		  pgen = TOPATH(&thiselem)->plist + epp->pdata.pathpt[0];
-		  if (ELEMENTTYPE(*pgen) == POLYGON) {
-		     setpt = TOPOLY(pgen)->points + k;
-		     setpt->y = ival;
+		  if (k < 0) {
+		     pgen = TOPATH(&thiselem)->plist;
+		     if (ELEMENTTYPE(*pgen) == POLYGON) {
+		        setpt = TOPOLY(pgen)->points;
+		        diff = ival - setpt->y;
+		     }
+		     else { /* spline */
+		        diff = ival - TOSPLINE(pgen)->ctrl[0].y;
+		     }
+		     for (pgen = TOPATH(&thiselem)->plist; pgen <
+				TOPATH(&thiselem)->plist +
+				TOPATH(&thiselem)->parts; pgen++) {
+		        if (ELEMENTTYPE(*pgen) == POLYGON) {
+			   for (i = 0; i < TOPOLY(pgen)->number; i++) {
+		              setpt = TOPOLY(pgen)->points + i;
+		              setpt->y += diff;
+			   }
+		        }
+		        else { /* spline */
+			   for (i = 0; i < 4; i++) {
+		              TOSPLINE(pgen)->ctrl[i].y += diff;
+			   }
+		           if (needrecalc) *needrecalc = True;
+		        }
+		     }
 		  }
-		  else { /* spline */
-		     TOSPLINE(pgen)->ctrl[k].y = ival;
-		     if (needrecalc) *needrecalc = True;
+		  else {
+		     pgen = TOPATH(&thiselem)->plist + epp->pdata.pathpt[0];
+		     if (ELEMENTTYPE(*pgen) == POLYGON) {
+		        setpt = TOPOLY(pgen)->points + k;
+		        setpt->y = ival;
+		     }
+		     else { /* spline */
+		        TOSPLINE(pgen)->ctrl[k].y = ival;
+		        if (needrecalc) *needrecalc = True;
+		     }
 		  }
 		  break;
 	       case POLYGON:
-	          setpt = TOPOLY(&thiselem)->points + k;
-		  setpt->y = ival;
+		  if (k < 0) {
+		     setpt = TOPOLY(&thiselem)->points;
+		     diff = ival - setpt->y;
+		     for (i = 0; i < TOPOLY(&thiselem)->number; i++) {
+			setpt = TOPOLY(&thiselem)->points + i;
+			setpt->y += diff;
+		     }
+		  }
+		  else {
+	             setpt = TOPOLY(&thiselem)->points + k;
+		     setpt->y = ival;
+		  }
 		  break;
 	       case SPLINE:
-		  TOSPLINE(&thiselem)->ctrl[k].y = ival;
+		  if (k < 0) {
+		     setpt = &(TOSPLINE(&thiselem)->ctrl[0]);
+		     diff = ival - setpt->y;
+		     for (i = 0; i < 4; i++) {
+			setpt = &(TOSPLINE(&thiselem)->ctrl[i]);
+			setpt->y += diff;
+		     }
+		  }
+		  else {
+		     TOSPLINE(&thiselem)->ctrl[k].y = ival;
+		  }
 		  if (needrecalc) *needrecalc = True;
 		  break;
 	       case LABEL:
@@ -969,6 +1069,7 @@ void pwriteback(objinstptr thisinst)
 		  k = epp->pdata.pathpt[1];
 	       else
 	          k = epp->pdata.pointno;
+	       if (k < 0) k = 0;
                switch(ops->which) {
 	          case P_POSITION_X:
 	             switch(thiselem->type) {
@@ -989,7 +1090,10 @@ void pwriteback(objinstptr thisinst)
 		           wtemp.ival = TOSPLINE(eptr)->ctrl[k].x;
 		           break;
 			case PATH:
-			   pgen = TOPATH(eptr)->plist + epp->pdata.pathpt[0];
+			   if (epp->pdata.pathpt[0] < 0)
+			      pgen = TOPATH(eptr)->plist;
+			   else
+			      pgen = TOPATH(eptr)->plist + epp->pdata.pathpt[0];
 			   if (ELEMENTTYPE(*pgen) == POLYGON) {
 	                      setpt = TOPOLY(pgen)->points + k;
 		              wtemp.ival = setpt->x;
@@ -1018,7 +1122,10 @@ void pwriteback(objinstptr thisinst)
 		           wtemp.ival = TOSPLINE(eptr)->ctrl[k].y;
 		           break;
 			case PATH:
-			   pgen = TOPATH(eptr)->plist + epp->pdata.pathpt[0];
+			   if (epp->pdata.pathpt[0] < 0)
+			      pgen = TOPATH(eptr)->plist;
+			   else
+			      pgen = TOPATH(eptr)->plist + epp->pdata.pathpt[0];
 			   if (ELEMENTTYPE(*pgen) == POLYGON) {
 	                      setpt = TOPOLY(pgen)->points + k;
 		              wtemp.ival = setpt->y;
@@ -1532,12 +1639,12 @@ char *getnumericalpkey(u_int mode)
 
 void makenumericalp(genericptr *gelem, u_int mode, char *key, short cycle)
 {
-   genericptr pgen;
+   genericptr pgen, *pathpgen;
    oparamptr ops, newops;
    eparamptr epp;
    XPoint *pptr;
    char new_key[7], *keyptr;
-   int pidx;
+   int pidx, i;
    short loccycle = cycle;
 
    /* Parameterized strings are handled by makeparam() */
@@ -1552,8 +1659,19 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key, short cycle)
    for (epp = (*gelem)->passed; epp != NULL; epp = epp->next) {
       ops = match_param(topobject, epp->key);
       if (ops->which == (u_char)mode) {
-	 Fprintf(stderr, "Cannot duplicate a parameter!\n");
-	 return;
+	 if ((mode == P_POSITION_X || mode == P_POSITION_Y) &&
+		((*gelem)->type == POLYGON || (*gelem)->type == SPLINE) &&
+		(TOPOLY(gelem)->cycle != NULL))
+	 {
+	    if ((cycle < 0) || (TOPOLY(gelem)->cycle->number != cycle)) {
+	       Fprintf(stderr, "Cannot duplicate a point parameter!\n");
+	       return;
+	    }
+	 }
+	 else {
+	    Fprintf(stderr, "Cannot duplicate a parameter!\n");
+	    return;
+	 }
       }
    }
 
@@ -1698,13 +1816,31 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key, short cycle)
 			TOPOLY(gelem)->cycle->number : -1;
 	    switch(mode) {
 	       case P_POSITION_X:
-	          pptr = TOPOLY(gelem)->points + loccycle;
-	          newops->parameter.ivalue = (int)pptr->x;
+		  if (loccycle == -1) {
+	             pptr = TOPOLY(gelem)->points;
+	             newops->parameter.ivalue = (int)pptr->x;
+		     for (i = 0; i < TOPOLY(gelem)->number; i++) {
+	                pptr = TOPOLY(gelem)->points + i;
+			pptr->x -= newops->parameter.ivalue;
+		     }
+		  } else {
+	             pptr = TOPOLY(gelem)->points + loccycle;
+	             newops->parameter.ivalue = (int)pptr->x;
+		  }
 	          epp->pdata.pointno = loccycle;
 	          break;
 	       case P_POSITION_Y:
-	          pptr = TOPOLY(gelem)->points + loccycle;
-	          newops->parameter.ivalue = (int)pptr->y;
+		  if (loccycle == -1) {
+	             pptr = TOPOLY(gelem)->points;
+	             newops->parameter.ivalue = (int)pptr->y;
+		     for (i = 0; i < TOPOLY(gelem)->number; i++) {
+	                pptr = TOPOLY(gelem)->points + i;
+			pptr->y -= newops->parameter.ivalue;
+		     }
+		  } else {
+	             pptr = TOPOLY(gelem)->points + loccycle;
+	             newops->parameter.ivalue = (int)pptr->y;
+		  }
 	          epp->pdata.pointno = loccycle;
 	          break;
 	       case P_STYLE:
@@ -1722,13 +1858,31 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key, short cycle)
 			TOSPLINE(gelem)->cycle->number : -1;
 	    switch(mode) {
 	       case P_POSITION_X:
-	          pptr = TOSPLINE(gelem)->ctrl + loccycle;
-	          newops->parameter.ivalue = (int)pptr->x;
+		  if (loccycle == -1) {
+	             pptr = &(TOSPLINE(gelem)->ctrl[0]);
+	             newops->parameter.ivalue = (int)pptr->x;
+		     for (i = 0; i < 4; i++) {
+	                pptr = &(TOSPLINE(gelem)->ctrl[i]);
+			pptr->x -= newops->parameter.ivalue;
+		     }
+		  } else {
+	             pptr = TOSPLINE(gelem)->ctrl + loccycle;
+	             newops->parameter.ivalue = (int)pptr->x;
+		  }
 	          epp->pdata.pointno = loccycle;
 	          break;
 	       case P_POSITION_Y:
-	          pptr = TOSPLINE(gelem)->ctrl + loccycle;
-	          newops->parameter.ivalue = (int)pptr->y;
+		  if (loccycle == -1) {
+	             pptr = &(TOSPLINE(gelem)->ctrl[0]);
+	             newops->parameter.ivalue = (int)pptr->y;
+		     for (i = 0; i < 4; i++) {
+	                pptr = &(TOSPLINE(gelem)->ctrl[i]);
+			pptr->y -= newops->parameter.ivalue;
+		     }
+		  } else {
+	             pptr = TOSPLINE(gelem)->ctrl + loccycle;
+	             newops->parameter.ivalue = (int)pptr->y;
+		  }
 	          epp->pdata.pointno = loccycle;
 	          break;
 	       case P_STYLE:
@@ -1765,21 +1919,79 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key, short cycle)
 	          break;
 	       case P_POSITION_X:
 		  newops->type = XC_INT;
-		  if (ELEMENTTYPE(pgen) == POLYGON)
-		     newops->parameter.ivalue = ((polyptr)pgen)->points[loccycle].x;
-		  else
-		     newops->parameter.ivalue = ((splineptr)pgen)->ctrl[loccycle].x;
-	          epp->pdata.pathpt[1] = loccycle;
-	          epp->pdata.pathpt[0] = pidx;
+		  if (loccycle == -1) {
+		     pathpgen = TOPATH(gelem)->plist;
+		     if (ELEMENTTYPE(*pathpgen) == POLYGON) {
+	                pptr = TOPOLY(pathpgen)->points;
+	                newops->parameter.ivalue = (int)pptr->x;
+		     }
+		     else {
+	                pptr = &(TOSPLINE(pathpgen)->ctrl[0]);
+	                newops->parameter.ivalue = (int)pptr->x;
+		     }
+		     for (pathpgen = TOPATH(gelem)->plist; pathpgen <
+				TOPATH(gelem)->plist + TOPATH(gelem)->parts;
+				pathpgen++) {
+			if (ELEMENTTYPE(*pathpgen) == POLYGON) {
+			   for (i = 0; i < TOPOLY(pathpgen)->number; i++) {
+	                      pptr = TOPOLY(pathpgen)->points + i;
+			      pptr->x -= newops->parameter.ivalue;
+			   }
+		 	}
+			else {
+			   for (i = 0; i < 4; i++) {
+	                      pptr = &(TOSPLINE(pathpgen)->ctrl[i]);
+			      pptr->x -= newops->parameter.ivalue;
+			   }
+			}
+		     }	
+		  }
+		  else {
+		     if (ELEMENTTYPE(pgen) == POLYGON)
+		        newops->parameter.ivalue = ((polyptr)pgen)->points[loccycle].x;
+		     else
+		        newops->parameter.ivalue = ((splineptr)pgen)->ctrl[loccycle].x;
+	             epp->pdata.pathpt[1] = loccycle;
+	             epp->pdata.pathpt[0] = pidx;
+		  }
 		  break;
 	       case P_POSITION_Y:
 		  newops->type = XC_INT;
-		  if (ELEMENTTYPE(pgen) == POLYGON)
-		     newops->parameter.ivalue = ((polyptr)pgen)->points[loccycle].y;
-		  else
-		     newops->parameter.ivalue = ((splineptr)pgen)->ctrl[loccycle].y;
-	          epp->pdata.pathpt[1] = loccycle;
-	          epp->pdata.pathpt[0] = pidx;
+		  if (loccycle == -1) {
+		     pathpgen = TOPATH(gelem)->plist;
+		     if (ELEMENTTYPE(*pathpgen) == POLYGON) {
+	                pptr = TOPOLY(pathpgen)->points;
+	                newops->parameter.ivalue = (int)pptr->y;
+		     }
+		     else {
+	                pptr = &(TOSPLINE(pathpgen)->ctrl[0]);
+	                newops->parameter.ivalue = (int)pptr->y;
+		     }
+		     for (pathpgen = TOPATH(gelem)->plist; pathpgen <
+				TOPATH(gelem)->plist + TOPATH(gelem)->parts;
+				pathpgen++) {
+			if (ELEMENTTYPE(*pathpgen) == POLYGON) {
+			   for (i = 0; i < TOPOLY(pathpgen)->number; i++) {
+	                      pptr = TOPOLY(pathpgen)->points + i;
+			      pptr->y -= newops->parameter.ivalue;
+			   }
+		 	}
+			else {
+			   for (i = 0; i < 4; i++) {
+	                      pptr = &(TOSPLINE(pathpgen)->ctrl[i]);
+			      pptr->y -= newops->parameter.ivalue;
+			   }
+			}
+		     }	
+		  }
+		  else {
+		     if (ELEMENTTYPE(pgen) == POLYGON)
+		        newops->parameter.ivalue = ((polyptr)pgen)->points[loccycle].y;
+		     else
+		        newops->parameter.ivalue = ((splineptr)pgen)->ctrl[loccycle].y;
+	             epp->pdata.pathpt[1] = loccycle;
+	             epp->pdata.pathpt[0] = pidx;
+		  }
 		  break;
 	    }
 	    break;
@@ -2286,7 +2498,9 @@ void parameterize(int mode, char *key, short cycle)
 {
    short *fselect, ptype;
    labelptr settext;
+   Boolean preselected;
 
+   preselected = (areawin->selects > 0) ? TRUE : FALSE;
    if (mode >= 0) {
       ptype = (short)param_select[mode];
       if (!checkselect(ptype)) select_element(ptype);
@@ -2309,7 +2523,7 @@ void parameterize(int mode, char *key, short cycle)
       else
 	 makenumericalp(topobject->plist + (*fselect), mode, key, cycle);
    }
-   unselect_all();
+   if (!preselected) unselect_all();
    setparammarks(NULL);
 }
 
