@@ -536,6 +536,17 @@ static char *nonprint[] = {
 	"Font", "Scale", "Color", "Kern", 
         "Parameter", ">", "Net_Name", "Error", NULL}; /* (jdk) */
 
+/* LaTeX translations of (some) xcircuit text escapes	*/
+/* Added by Fabian Inostroza, July 14, 2013		*/
+
+static char *nonprinttex[] = {
+	"", "_{", "^{", "}",
+	"\\underline{", "\\overline{", "}",
+	"Tab_Stop", "Tab_Forward", "Tab_Backward",
+	"Halfspace", "Quarterspace", "<Return>",
+	"Font", "Scale", "Color", "Kern", 
+        "Parameter", ">", "Net_Name", "Error", NULL};
+
 /*----------------------------------------------------------------------*/
 /* charprint():  							*/
 /* Write a printable version of the character or command at the 	*/
@@ -580,6 +591,42 @@ void charprint(char *sout, stringpart *strptr, int locpos)
    }
 }
 
+/*----------------------------------------------*/
+/* Same as above, but uses LaTeX translations	*/
+/*----------------------------------------------*/
+
+void charprinttex(char *sout, stringpart *strptr, int locpos)
+{
+   char sc;
+ 
+   switch (strptr->type) {
+      case TEXT_STRING:
+	 if (strptr->data.string) {
+            if (locpos > strlen(strptr->data.string)) {
+	       strcpy(sout, "<ERROR>");
+	    }
+            else sc = *(strptr->data.string + locpos);
+            if (isprint(sc))
+	       sprintf(sout, "%c", sc);
+            else
+	       sprintf(sout, "/%03o", (u_char)sc);
+	 }
+	 else
+	    *sout = '\0';
+	 break;
+      case FONT_NAME:
+	 sprintf(sout, "");
+	 break;
+      case FONT_SCALE:
+      case KERN:
+      case PARAM_START:
+	 break;
+      default:
+         strcpy(sout, nonprinttex[strptr->type]);
+	 break;
+   }
+}
+
 /*----------------------------------------------------------------------*/
 /* Print a string (allocates memory for the string; must be freed by	*/
 /* the calling routine).						*/
@@ -609,6 +656,27 @@ char *xcstringtostring(stringpart *strtop, objinstptr localinst, Boolean textonl
    return sout;
 }
 
+/*----------------------------------------------*/
+/* Same as above, but uses LaTeX translations	*/
+/*----------------------------------------------*/
+
+char *textprinttex(stringpart *strtop, objinstptr localinst)
+{
+   stringpart *strptr;
+   int pos = 0, locpos;
+   char *sout;
+
+   sout = (char *)malloc(1);
+   sout[0] = '\0';
+
+   while ((strptr = findstringpart(pos++, &locpos, strtop, localinst)) != NULL) {
+      charprinttex(_STR, strptr, locpos);
+      sout = (char *)realloc(sout, strlen(sout) + strlen(_STR) + 1);
+      strcat(sout, _STR);
+   }
+   return sout;
+}
+
 /*----------------------------------------------------------------------*/
 /* Wrappers for xcstringtostring():					*/
 /*  stringprint() includes information on text controls appropriate	*/
@@ -1953,9 +2021,8 @@ void composefontlib(short cfont)
       instancedefaults(*drawinst, libobj,
 		(i % 16) * del + qdel,		/* X position */
 		-(i / 16) * del + qdel);	/* Y position */
-      drawinst = (objinstptr *)directory->plist + directory->parts;
+      drawinst = (objinstptr *)directory->plist + directory->parts - 1;
       (*drawinst)->color = DEFAULTCOLOR;
-      directory->parts++;
    }
 
    /* separate characters with gridlines (17 vert., 17 horiz.) */
@@ -1984,7 +2051,6 @@ void composefontlib(short cfont)
          pointptr->x = 16 * del;
          pointptr->y = (17 - i) * del;
       }
-      directory->parts++;
    }
 
    /* Set the bounding box for this display. 				*/
